"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deactivate = exports.activate = exports.countTotalDuration = exports.countDialogueCharacters = void 0;
const vscode = require("vscode");
const yaml = require("js-yaml");
const fs = require("fs");
const path = require("path");
const os = require("os");
///// 文字数カウント
async function countDialogueCharacters() {
    const editor = vscode.window.activeTextEditor;
    if (!editor) {
        vscode.window.showErrorMessage('ファイルを開いてください。');
        return;
    }
    const text = editor.document.getText();
    let yamlData;
    try {
        yamlData = yaml.load(text);
    }
    catch (err) {
        vscode.window.showErrorMessage('YAMLのパースに失敗しました。');
        return;
    }
    if (!yamlData?.sequences) {
        vscode.window.showErrorMessage('YAML構造が不正です。sequencesが見つかりません。');
        return;
    }
    // 換算基準
    const durationCount = 5; // 5文字＝1秒（仮）
    // 全シーケンスID、シーンIDの範囲を取得
    const sequenceIds = yamlData.sequences.map((seq) => seq.id);
    const minSequenceId = Math.min(...sequenceIds);
    const maxSequenceId = Math.max(...sequenceIds);
    // まずユーザーから範囲入力を受け取る
    const sequenceRangeInput = await vscode.window.showInputBox({ prompt: "シーケンス範囲を入力（例: 1-3、2-、-5、-）" });
    const sceneRangeInput = await vscode.window.showInputBox({ prompt: "シーン範囲を入力（例: 1-5、2-、-4、-）" });
    let sequenceRange, sceneRange;
    try {
        sequenceRange = parseRangeInput(sequenceRangeInput, minSequenceId, maxSequenceId);
        sceneRange = parseRangeInput(sceneRangeInput, 1, 9999); // シーンはとりあえず1以上9999以下と想定
    }
    catch (e) {
        vscode.window.showErrorMessage(e.message);
        return;
    }
    let totalCharacters = 0;
    for (const sequence of yamlData.sequences ?? []) {
        if (sequence.id >= sequenceRange.start && sequence.id <= sequenceRange.end) {
            for (const scene of sequence.scenes ?? []) {
                if (scene.id >= sceneRange.start && scene.id <= sceneRange.end) {
                    for (const cut of scene.cuts ?? []) {
                        for (const dialogue of cut.dialogue ?? []) {
                            if (dialogue.text) {
                                totalCharacters += dialogue.text.length;
                            }
                        }
                    }
                }
            }
        }
    }
    const estimatedDuration = (totalCharacters / durationCount).toFixed(1);
    vscode.window.showInformationMessage(`セリフの合計文字数: ${totalCharacters}文字 (推定尺: 約${estimatedDuration}秒)`);
}
exports.countDialogueCharacters = countDialogueCharacters;
///// --- 総尺カウントコマンド
async function countTotalDuration() {
    const editor = vscode.window.activeTextEditor;
    if (!editor) {
        vscode.window.showErrorMessage('ファイルを開いてください。');
        return;
    }
    const text = editor.document.getText();
    let yamlData;
    try {
        yamlData = yaml.load(text);
    }
    catch (err) {
        vscode.window.showErrorMessage('YAMLのパースに失敗しました。');
        return;
    }
    if (!yamlData?.sequences) {
        vscode.window.showErrorMessage('YAML構造が不正です。sequencesが見つかりません。');
        return;
    }
    // 全シーケンスIDの範囲取得
    const sequenceIds = yamlData.sequences.map((seq) => seq.id);
    const minSequenceId = Math.min(...sequenceIds);
    const maxSequenceId = Math.max(...sequenceIds);
    // ユーザーに範囲入力を求める
    const sequenceRangeInput = await vscode.window.showInputBox({ prompt: "シーケンス範囲を入力（例: 1-3、2-、-5、-）" });
    const sceneRangeInput = await vscode.window.showInputBox({ prompt: "シーン範囲を入力（例: 1-5、2-、-4、-）" });
    let sequenceRange, sceneRange;
    try {
        sequenceRange = parseRangeInput(sequenceRangeInput, minSequenceId, maxSequenceId);
        sceneRange = parseRangeInput(sceneRangeInput, 1, 9999);
    }
    catch (e) {
        vscode.window.showErrorMessage(e.message);
        return;
    }
    let totalDuration = 0;
    for (const sequence of yamlData.sequences ?? []) {
        if (sequence.id >= sequenceRange.start && sequence.id <= sequenceRange.end) {
            for (const scene of sequence.scenes ?? []) {
                if (scene.id >= sceneRange.start && scene.id <= sceneRange.end) {
                    for (const cut of scene.cuts ?? []) {
                        if (typeof cut.duration === 'number') {
                            totalDuration += cut.duration;
                        }
                    }
                }
            }
        }
    }
    const formattedDuration = formatDuration(totalDuration);
    vscode.window.showInformationMessage(`総尺: ${totalDuration}秒（${formattedDuration}）`);
}
exports.countTotalDuration = countTotalDuration;
///// --- 秒数→分秒フォーマット関数
function formatDuration(seconds) {
    const minutes = Math.floor(seconds / 60);
    const remainingSeconds = seconds % 60;
    return `${minutes}分${remainingSeconds}秒`;
}
///// 範囲指定
function parseRangeInput(input, min, max) {
    if (!input || input.trim() === '-') {
        // 空、または"-"なら全範囲
        return { start: min, end: max };
    }
    const parts = input.split('-');
    if (parts.length !== 2) {
        throw new Error('範囲入力が不正です。例: 1-5、-3、5-、-');
    }
    const startStr = parts[0].trim();
    const endStr = parts[1].trim();
    const start = startStr === '' ? min : parseInt(startStr);
    const end = endStr === '' ? max : parseInt(endStr);
    if (isNaN(start) || isNaN(end)) {
        throw new Error('範囲入力が数値ではありません。');
    }
    if (start > end) {
        throw new Error('開始が終了より大きくなっています。');
    }
    return { start, end };
}
function activate(context) {
    ///// Add NewFile
    context.subscriptions.push(vscode.commands.registerCommand('sp.createNewFile', async () => {
        const now = new Date();
        const timestamp = now.getTime();
        const formattedDate = now.toISOString().split('T')[0];
        // 作品タイトル入力
        const title = await vscode.window.showInputBox({
            prompt: '新しい作品のタイトルを入力してください',
            placeHolder: '例: 放課後、そして未来へ',
            value: '新しい作品タイトル'
        });
        if (title === undefined) {
            return;
        }
        // 作者名入力（カンマ区切り）
        const authorsInput = await vscode.window.showInputBox({
            prompt: '作者名をカンマ区切りで入力してください（例: 山田太郎, 佐藤花子）',
            placeHolder: '作者名を入力（複数可）',
            value: '作者名'
        });
        if (authorsInput === undefined) {
            return;
        }
        // 作者リストへ変換
        const authors = authorsInput.split(',')
            .map(author => author.trim())
            .filter(author => author.length > 0);
        const authorsYaml = authors.map(author => `  - "${author}"`).join('\n');
        // YAML初期テンプレート
        const initialContent = `title: "${title}"
authors:
${authorsYaml}

created_at: "${formattedDate}"
updated_at: "${formattedDate}"
version: "1.0"
description: |
  ここに作品の説明を書いてください。
repository: "https://github.com/yourname/yourproject"

sequences:
  - id: 1
    name: "最初のシーケンス"
    description: ""
    scenes:
      - id: 1
        name: "最初のシーン"
        duration: 
        time_of_day: "昼"
        se: 
        description: ""
        actors: 
        cuts: 
          - id: 1
            camera: "カメラの説明"
            description: "カット内容"
            se: 
            dialogue: 
            duration: 3
            transition: 
`;
        // ここからが正式版処理
        const uri = vscode.Uri.parse(`untitled:新しいシナリオ-${timestamp}.sp.yml`);
        const edit = new vscode.WorkspaceEdit();
        edit.createFile(uri, { ignoreIfExists: true });
        await vscode.workspace.applyEdit(edit);
        const document = await vscode.workspace.openTextDocument(uri);
        const editor = await vscode.window.showTextDocument(document, { preview: false });
        // 初期コンテンツを書き込む
        await editor.edit(editBuilder => {
            editBuilder.insert(new vscode.Position(0, 0), initialContent);
        });
    }));
    ///// Add Template
    context.subscriptions.push(vscode.commands.registerCommand('sp.addTemplate', async () => {
        const editor = vscode.window.activeTextEditor;
        if (!editor) {
            vscode.window.showErrorMessage('エディターが開かれていません。');
            return;
        }
        const template = `
title: "タイトルをここに"
authors:
  - "作者"

created_at: "制作日"
updated_at: "修正日"
version: "バージョン"
description: |
  ここに作品の説明を書いてください。
repository: "https://github.com/yourname/yourproject"

sequences:
  - id: 1
    name: "最初のシーケンス"
    description: ""
    scenes:
      - id: 1
        name: "最初のシーン"
        duration: 
        time_of_day: "昼"
        se: 
        description: ""
        actors: 
        cuts: 
          - id: 1
            camera: "カメラの説明"
            description: "カット内容"
            se: 
            dialogue: 
            duration: 3
            transition: 
`;
        const position = editor.selection.active;
        await editor.edit(editBuilder => {
            editBuilder.insert(position, template);
        });
    }));
    ///// Add Sequences
    context.subscriptions.push(vscode.commands.registerCommand('sp.addSequences', async () => {
        const editor = vscode.window.activeTextEditor;
        if (!editor) {
            vscode.window.showErrorMessage('エディターが開かれていません。');
            return;
        }
        const position = editor.selection.active;
        const lineText = editor.document.lineAt(position.line).text;
        const baseIndent = ' '.repeat(2); // 2スペース
        const snippet = `
${baseIndent}- id: 
${baseIndent}  name: ""
${baseIndent}  description: ""
${baseIndent}  scenes: 
  `;
        await editor.edit(editBuilder => {
            editBuilder.insert(position, snippet);
        });
    }));
    ///// Add Scene
    context.subscriptions.push(vscode.commands.registerCommand('sp.addScene', async () => {
        const editor = vscode.window.activeTextEditor;
        if (!editor) {
            vscode.window.showErrorMessage('アクティブなドキュメントがありません');
            return;
        }
        const position = editor.selection.active;
        const newPosition = position.with(position.line, 0);
        editor.selection = new vscode.Selection(newPosition, newPosition);
        const baseIndent = ' '.repeat(6); // 6スペース
        const snippet = `
${baseIndent}- id: 
${baseIndent}  name: "新しいシーン"
${baseIndent}  duration: 
${baseIndent}  time_of_day: "昼"
${baseIndent}  se: 
${baseIndent}  description: ""
${baseIndent}  actors: 
${baseIndent}  cuts:
${baseIndent}    - id: 
${baseIndent}      camera: ""
${baseIndent}      description: ""
${baseIndent}      se: 
${baseIndent}      dialogue: 
${baseIndent}      duration: 3
${baseIndent}      transition: 
`;
        editor.insertSnippet(new vscode.SnippetString(snippet), editor.selection.active);
    }));
    ///// Add Cut
    context.subscriptions.push(vscode.commands.registerCommand('sp.addCut', async () => {
        const editor = vscode.window.activeTextEditor;
        if (!editor) {
            vscode.window.showErrorMessage('アクティブなドキュメントがありません');
            return;
        }
        const position = editor.selection.active;
        const newPosition = position.with(position.line, 0);
        editor.selection = new vscode.Selection(newPosition, newPosition);
        // 固定インデントでスニペット作成（例：10スペース）
        const baseIndent = ' '.repeat(10);
        const snippet = `
${baseIndent}- id: 
${baseIndent}  camera: ""
${baseIndent}  description: ""
${baseIndent}  se: 
${baseIndent}  dialogue: 
${baseIndent}  duration: 3
${baseIndent}  transition: 
`;
        editor.insertSnippet(new vscode.SnippetString(snippet), newPosition);
    }));
    ///// Count Total Duration
    context.subscriptions.push(vscode.commands.registerCommand('sp.countTotalDuration', countTotalDuration));
    ///// Count Words
    context.subscriptions.push(vscode.commands.registerCommand('sp.countDialogueCharacters', countDialogueCharacters));
    ///// Export Html&CSS
    context.subscriptions.push(vscode.commands.registerCommand('sp.exportHtml', async () => {
        const editor = vscode.window.activeTextEditor;
        if (!editor) {
            vscode.window.showErrorMessage('出力対象が選択されていません');
            return;
        }
        const document = editor.document;
        const text = document.getText();
        // YAML読み込み & 型アサーション
        let yamlContent;
        try {
            yamlContent = yaml.load(text);
        }
        catch (e) {
            vscode.window.showErrorMessage('YAMLのパースエラー: ' + e.message);
            return;
        }
        if (!yamlContent) {
            vscode.window.showErrorMessage('YAMLが空です。');
            return;
        }
        // レイアウト選択
        const layout = await vscode.window.showQuickPick(["縦書き（脚本風）", "横書き（モダン）", "印刷用（白黒）"], { placeHolder: "レイアウトスタイルを選択してください" });
        if (!layout) {
            vscode.window.showInformationMessage("出力がキャンセルされました。");
            return;
        }
        let cssFileName = "style-vertical.css";
        if (layout.includes("横書き"))
            cssFileName = "style-horizontal.css";
        if (layout.includes("印刷用"))
            cssFileName = "style-print.css";
        // ファイル名生成（安全なタイトル変換）
        const safeTitle = (yamlContent.title || "scenario")
            .replace(/\s+/g, "_")
            .replace(/[\\/:*?"<>|]/g, "_");
        const defaultFileName = `${safeTitle}.html`;
        const workspaceFolder = vscode.workspace.workspaceFolders?.[0]?.uri.fsPath || os.homedir();
        const uri = await vscode.window.showSaveDialog({
            defaultUri: vscode.Uri.file(path.join(workspaceFolder, defaultFileName)),
            filters: { 'HTML Files': ['html'] }
        });
        if (!uri) {
            vscode.window.showInformationMessage('書き出しがキャンセルされました。');
            return;
        }
        const styleUri = vscode.Uri.file(path.join(path.dirname(uri.fsPath), 'style.css'));
        const html = generateHtml(yamlContent, styleUri);
        fs.writeFileSync(uri.fsPath, html, 'utf8');
        const extensionPath = context.extensionPath;
        const cssSourcePath = path.join(extensionPath, 'media', cssFileName);
        const cssTargetPath = path.join(path.dirname(uri.fsPath), 'style.css');
        fs.copyFile(cssSourcePath, cssTargetPath, async (err) => {
            if (err) {
                vscode.window.showErrorMessage('CSSコピーに失敗しました。');
                return;
            }
            vscode.window.showInformationMessage(`HTMLとCSS（${layout}）を書き出しました。`);
            await vscode.env.openExternal(uri);
        });
    }));
    ///// Preview
    context.subscriptions.push(vscode.commands.registerCommand('sp.previewScene', () => {
        previewScene(context);
    }));
    vscode.window.showInformationMessage('ScreenPlay Formater 拡張機能が有効になりました！');
}
exports.activate = activate;
function deactivate() { }
exports.deactivate = deactivate;
function previewScene(context) {
    const editor = vscode.window.activeTextEditor;
    if (!editor) {
        vscode.window.showErrorMessage("アクティブなドキュメントがありません");
        return;
    }
    const text = editor.document.getText();
    let parsed;
    try {
        parsed = yaml.load(text);
    }
    catch (err) {
        vscode.window.showErrorMessage("YAMLパースに失敗しました");
        return;
    }
    const panel = vscode.window.createWebviewPanel("scenePreview", "シナリオプレビュー", vscode.ViewColumn.Beside, {});
    // @ts-ignore
    const styleUri = panel.webview.asWebviewUri(
    // @ts-ignore
    vscode.Uri.joinPath(context.extensionUri, 'media', 'style.css'));
    const html = generateHtml(parsed, styleUri);
    panel.webview.html = html;
}
///// HTML生成関数
function generateHtml(data, styleUri) {
    if (!data || !data.sequences) {
        return "<h2>シナリオが正しく構造化されていません</h2>";
    }
    let html = `
  <html>
  <head>
    <meta charset="UTF-8" />
    <link href="${styleUri}" rel="stylesheet" />
  </head>
  <body>
  `;
    html += `<div class="scenario-header"><div class="scenario-title">${data.title || "（タイトル未設定）"}</div>`;
    html += `<div class="scenario-meta">`;
    if (data.authors) {
        html += `<div><strong>作者:</strong> ${Array.isArray(data.authors) ? data.authors.join(", ") : data.authors}</div>`;
    }
    if (data.created_at) {
        html += `<div><strong>作成日:</strong> ${data.created_at}</div>`;
    }
    if (data.updated_at) {
        html += `<div><strong>更新日:</strong> ${data.updated_at}</div>`;
    }
    if (data.version) {
        html += `<div><strong>バージョン:</strong> ${data.version}</div>`;
    }
    if (data.description) {
        html += `<div><strong>説明:</strong><br>${data.description.replace(/\n/g, "<br>")}</div>`;
    }
    if (data.repository) {
        html += `<div><strong>リポジトリ:</strong> <a href="${data.repository}" target="_blank">${data.repository}</a></div>`;
    }
    html += `</div></div>`;
    for (const seq of data.sequences) {
        html += `<div class="sequence" data-id="${seq.id}">
      <div class="sequence-header"><span class="label">シーケンス</span>${seq.id} ${seq.name}</div>
      <div class="sequence-description">${seq.description || ""}</div>`;
        for (const scene of seq.scenes || []) {
            html += `<div class="scene" data-id="${scene.id}">
        <div class="scene-header"><span class="label">シーン</span>${scene.id} ${scene.name}</div>
        <div class="scene-description">${scene.description || ""}</div>
        <div class="scene-time">${scene.time_of_day || "時間未設定"}</div>`;
            for (const cut of scene.cuts || []) {
                html += `<div class="cut" data-id="${cut.id}">
          <div class="cut-header"><span class="label">カット</span>${cut.id}</div>
          <div class="camera"><span class="label">カメラ:</span>${cut.camera || ""}</div>
          <div class="cut-description"><span class="label">描写:</span>${cut.description || ""}</div>
          <div class="cut-duration"><span class="label">時間:</span>${cut.duration || "?"}秒</div>
          <div class="dialogue"><span class="label">セリフ:</span>`;
                for (const d of cut.dialogue || []) {
                    html += `<div><span class="speaker">${d.speaker}:</span> <span class="text">${d.text}</span> (${d.description || ""})</div>`;
                }
                html += `</div>
          <div class="transition"><span class="label">トランジション:</span>${cut.transition || ""}</div>
        </div>`;
            }
            html += `</div>`; // scene
        }
        html += `</div>`; // sequence
    }
    html += "</body></html>";
    return html;
}
function renderBlock(data, fields, className) {
    const parts = [`<div class="${className}">`];
    for (const key in fields) {
        parts.push(`<div class="${key}">${fields[key]}: ${data[key] ?? ""}</div>`);
    }
    parts.push(`</div>`);
    return parts.join("\n");
}
//# sourceMappingURL=extension.js.map